public class Ausgangsfuzzyset
{
     private double[] stellen = null;
     private FuzzygroesseMitteAusgang[] fuzzygroesse = null;
     
     private double schwerpunkt = 0.0; //endergebnis
     private double gesamtflaeche = 0.0;
     private double gewichteteSummeDerStellen = 0.0;
     private int    anzahl_aufrufe=0;

     private double epsilon = 0.000001; //Minimal akzeptierte Flächensumme
     private double default_wert = 0.0; //Mußß vorberechnet werden im Konstruktor!!!!, z.B. Schwerp. für alle Ausgangsfuzzygroesse mit Akt. 1 
     
     public Ausgangsfuzzyset(double[] stellen)
     {
          if(stellen==null)
          {
              println("ERROR in Konstruktor Ausgangsfuzzyset: keine stellen");
              return;
          }    
          if(stellen.length<4)
          {
              println("ERROR in Konstruktor Ausgangsfuzzyset: weniger als 4 stellen");
              return;
          }    
          for(int i=0;i<stellen.length-1;i++)
          {
              if(stellen[i+1]<=stellen[i])
              {
                  println("ERROR in Konstruktor Eingangsfuzzyset: stellen nicht streng monoton ansteigend");
                  return;                   
              }
          }
          
          this.stellen = new double[stellen.length];
          for(int i=0;i<stellen.length;i++)
              this.stellen[i] = stellen[i];
              
          fuzzygroesse = new FuzzygroesseMitteAusgang[stellen.length-2];
          
          for(int i=0;i<fuzzygroesse.length;i++)
              fuzzygroesse[i] = new FuzzygroesseMitteAusgang(stellen,i);
//println("++++ Es wurden "+fuzzygroesse.length+" Fuzzygroessen für das Ausgangsfuzzyset erzeugt ++++");              
          //Für Ausgangsfuzzygroessen default_schwerpunkt als der, der sich ergibt, wenn alle Fuzzygroessen mit 1 aktiv sind:
          this.resetStellwert();
          for(int i=0;i<fuzzygroesse.length;i++)
              this.addAktivierung(i,1.0);
          default_wert = holeStellwert();    
     }
 
     //    Jede Regel liefert einen Index eines der Ausgangsfuzzygrößen und dessen Aktivierung
     //    Darum Methode in FuzzygroesseMitte nötig, um aus der Aktivierung Fläche und Schwerpunkt zu bestimmen!
     //    Übergabe an berechneStellwert: {{}}
     //    Neue Idee:
     //    1. RESET: Gesamtfläche Null setzen, Gewichtete Summe der Stellen Null setzen, Schwerpunktslage Null setzen
     //    2. Mehrmals addAktivierung(index,aktivierung) aktivierung==Höhe des Trapezes
     //             --- Ergebnis gewichtete Stellen aufsummieren, Flächen aufsummieren
     //    3. gewichtete Summen durch Gesamtfläche teilen == schwerpunkt == Ausgabe
     
     //1.
     public void resetStellwert()
     {
         schwerpunkt = 0.0; //endergebnis
         gesamtflaeche = 0.0;
         gewichteteSummeDerStellen = 0.0;
         anzahl_aufrufe=0;
     }
     //2.
     public void addAktivierung(int index, double aktivierung)
     {
//println("INDEX="+index+" akt="+aktivierung+" fuzzygroesse.length="+fuzzygroesse.length);
         if(aktivierung>epsilon)
         {
             double[] flaeche_schwerpunkt = fuzzygroesse[index].berechneTrapezflaecheUndSchwerpunkt(aktivierung);
             gesamtflaeche+=flaeche_schwerpunkt[0];
             gewichteteSummeDerStellen+=flaeche_schwerpunkt[0]*flaeche_schwerpunkt[1];
         }    
         anzahl_aufrufe++;
     }
     //3. 
     public double holeStellwert()
     {
          if(anzahl_aufrufe==0)
          {
                println("ERROR in Ausgangsfuzzyset: Es wurden gar keine Regelaktivierungen hinzugefügt.");
                return default_wert;
          }
          else if(gesamtflaeche<=epsilon) 
          {
                println("ERROR in Ausgangsfuzzyset: Es wurden gar keine Regelaktivierungen hinzugefügt.");
                return default_wert;
          }
          else
          {
                return gewichteteSummeDerStellen/gesamtflaeche;
          }
     }
}
